<?php
/**
 * @author jurgenhaas
 */

namespace GMJH\SQRL\Drupal7;

/**
 * An override class for Drupal 7 SQRL client operations
 *
 * @author jurgenhaas
 *
 * @link
 */
class Client extends \GMJH\SQRL\Client {

  /**
   * @return string
   */
  public function site_name() {
    return variable_get('site_name', $this->sqrl->get_base_url());
  }

  /**
   * @return string
   */
  private function cid() {
    return 'SQRL:SERVER:RESPONSE:' . $this->sqrl->get_nut();
  }

  /**
   * @param mixed $value
   */
  protected function save($value) {
    cache_set($this->cid(), $value, 'cache', $this->get_timeout());
  }

  /**
   * @return mixed
   */
  protected function load() {
    $cid = $this->cid();
    $cache = cache_get($cid, 'cache');
    if (empty($cache->data)) {
      // This is the initial request from the client, not a response to the server.
      return FALSE;
    }

    cache_clear_all($cid, 'cache');
    return $cache->data;
  }

  /**
   * @param SQRL $sqrl
   * @return Account
   */
  protected function get_link_account($sqrl) {
    return new Account(user_load($sqrl->get_operation_param('uid')));
  }

  /**
   * @param string $key
   * @return Account
   */
  protected function find_user_account($key) {
    // Make sure we find the enabled accounts first (order by status descending)
    // just in case that there are multiple accounts for the same ID.
    $uid = db_select('sqrl_user', 'u')
      ->fields('u', array('uid'))
      ->condition('u.idk', $this->hash_pubkey($key))
      ->orderBy('u.status', 'DESC')
      ->execute()
      ->fetchField();
    return empty($uid) ? FALSE : new Account(user_load($uid));
  }

  /**
   * @param string $key
   * @param bool $just_active
   * @return array
   */
  public function find_all_user_accounts($key, $just_active = TRUE) {
    $status = $just_active ? 1 : array(0, 1);
    return db_select('sqrl_user', 'u')
      ->fields('u', array('uid'))
      ->condition('u.idk', $this->hash_pubkey($key))
      ->condition('u.status', $status)
      ->execute()
      ->fetchAll();
  }

  /**
   * @return bool
   */
  public function user_account_register_allowed() {
    return (variable_get('user_register') > 0);
  }

  /**
   * Return the provided sqrl key in a hashed and salted format in which the
   * module will store this key in the database.
   *
   * @param string $key
   * @return string
   */
  public function hash_pubkey($key) {
    return drupal_hmac_base64($key, drupal_get_private_key() . drupal_get_hash_salt());
  }

}
