<?php

/**
 * @file
 * TBD.
 */

/**
 * Build a form for all system settings relevant for the SQRL module.
 *
 * @param array $form
 * @return array
 */
function sqrl_admin($form) {

  include_once 'sqrl.aes.inc';
  $sqrl = sqrl_get_instance();

  // Container for all connection related settings.
  $form['connection'] = array(
    '#type' => 'fieldset',
    '#title' => t('Connection'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );
  $form['connection']['sqrl_base_url'] = array(
    '#type' => 'textfield',
    '#title' => t('Base URL'),
    '#default_value' => $sqrl->get_base_url(),
    '#disabled' => TRUE,
  );
  $form['connection']['sqrl_insecure_connection_port'] = array(
    '#type' => 'textfield',
    '#title' => t('Port for insecure connection'),
    '#default_value' => variable_get('sqrl_insecure_connection_port', SQRL_DEFAULT_INSECURE_CONNECTION_PORT),
    '#size' => 4,
  );
  $form['connection']['secure_connection'] = array(
    '#type' => 'container',
    '#title' => t('Secure connection'),
    '#disabled' => !$sqrl->is_secure_connection_available(),
  );
  $form['connection']['secure_connection']['sqrl_use_secure_connection'] = array(
    '#type' => 'checkbox',
    '#title' => t('Use secure connection'),
    '#default_value' => variable_get('sqrl_use_secure_connection', SQRL_DEFAULT_USE_SECURE_CONNECTION),
  );
  $form['connection']['secure_connection']['sqrl_secure_connection_port'] = array(
    '#type' => 'textfield',
    '#title' => t('Port for secure connection'),
    '#default_value' => variable_get('sqrl_secure_connection_port', SQRL_DEFAULT_SECURE_CONNECTION_PORT),
    '#size' => 4,
  );

  // Container for encryption settings.
  $form['encryption'] = array(
    '#type' => 'fieldset',
    '#title' => t('Encryption'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );
  $form['encryption']['nut_query'] = array(
    '#type' => 'fieldset',
    '#title' => t('Encryption for the nut in URL'),
  );
  $aes_form = aes_config();
  $form['encryption']['nut_query']['sqrl_aes_implementation_query'] = $aes_form['aes_implementation'];
  $form['encryption']['nut_query']['sqrl_aes_implementation_query']['#default_value'] = sqrl_aes_get_implementation(FALSE);
  $form['encryption']['nut_query']['sqrl_aes_cipher_query'] = $aes_form['aes_cipher'];
  $form['encryption']['nut_query']['sqrl_aes_cipher_query']['#default_value'] = sqrl_aes_get_cipher(FALSE);
  $form['encryption']['nut_query']['sqrl_aes_key_query'] = $aes_form['aes_key'];
  $form['encryption']['nut_query']['sqrl_aes_key_query']['#default_value'] = sqrl_aes_get_key(FALSE);
  $form['encryption']['nut_cookie'] = array(
    '#type' => 'fieldset',
    '#title' => t('Encryption for the nut in Cookie'),
  );
  $form['encryption']['nut_cookie']['sqrl_aes_implementation_cookie'] = $aes_form['aes_implementation'];
  $form['encryption']['nut_cookie']['sqrl_aes_implementation_cookie']['#default_value'] = sqrl_aes_get_implementation(TRUE);
  $form['encryption']['nut_cookie']['sqrl_aes_cipher_cookie'] = $aes_form['aes_cipher'];
  $form['encryption']['nut_cookie']['sqrl_aes_cipher_cookie']['#default_value'] = sqrl_aes_get_cipher(TRUE);
  $form['encryption']['nut_cookie']['sqrl_aes_key_cookie'] = $aes_form['aes_key'];
  $form['encryption']['nut_cookie']['sqrl_aes_key_cookie']['#default_value'] = sqrl_aes_get_key(TRUE);

  // Container for all other options.
  $form['options'] = array(
    '#type' => 'fieldset',
    '#title' => t('Options'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );
  $form['options']['sqrl_mode'] = array(
    '#type' => 'radios',
    '#title' => t('Mode'),
    '#options' => array(
      0 => t('Optional'),
      1 => t('Exclusive'),
    ),
    '#default_value' => variable_get('sqrl_mode', SQRL_DEFAULT_MODE),
    '#description' => t('In exclusive mode, user accounts can only be used with SQRL and username/password authentication no longer works.'),
  );
  $form['options']['sqrl_watchdog_severity'] = array(
    '#type' => 'select',
    '#title' => t('Log level'),
    '#options' => watchdog_severity_levels(),
    '#default_value' => variable_get('sqrl_watchdog_severity', WATCHDOG_WARNING),
  );
  $form['options']['sqrl_watchdog_severity_ajax'] = array(
    '#type' => 'select',
    '#title' => t('Log level for Ajax calls'),
    '#options' => watchdog_severity_levels(),
    '#default_value' => variable_get('sqrl_watchdog_severity_ajax', WATCHDOG_WARNING),
  );
  $form['options']['sqrl_session_allow_multiple'] = array(
    '#type' => 'checkbox',
    '#title' => t('Allow multiple sessions per user account'),
    '#default_value' => variable_get('sqrl_session_allow_multiple', SQRL_DEFAULT_SESSION_ALLOW_MULTIPLE),
  );
  $form['options']['sqrl_account_allow_multiple_per_id'] = array(
    '#type' => 'checkbox',
    '#title' => t('Allow multiple user accounts for each SQRL ID'),
    '#default_value' => variable_get('sqrl_account_allow_multiple_per_id', SQRL_DEFAULT_ACCOUNT_ALLOW_MULTIPLE_PER_ID),
  );
  $form['options']['sqrl_id_allow_multiple_per_account'] = array(
    '#type' => 'checkbox',
    '#title' => t('Allow multiple SQRL IDs per user account'),
    '#default_value' => variable_get('sqrl_id_allow_multiple_per_account', SQRL_DEFAULT_ID_ALLOW_MULTIPLE_PER_ACCOUNT),
  );
  $form['options']['sqrl_accounts_name_pattern'] = array(
    '#type' => 'textfield',
    '#title' => t('Pattern for user names when creating them by SQRL'),
    '#default_value' => variable_get('sqrl_accounts_name_pattern', 'SQRL-[rand]'),
    '#description' => t('Use the token [rand] for the random bytes, if missing they get appended.'),
  );
  $form['options']['sqrl_accounts_name_random_length'] = array(
    '#type' => 'textfield',
    '#title' => t('Number of the random bytes'),
    '#size' => 6,
    '#default_value' => variable_get('sqrl_accounts_name_random_length', 5),
  );
  $form['options']['sqrl_qr_size'] = array(
    '#type' => 'textfield',
    '#title' => t('Size of the QR code'),
    '#size' => 6,
    '#default_value' => variable_get('sqrl_qr_size', SQRL_DEFAULT_IMAGE_SIZE),
  );
  $form['options']['sqrl_poll_interval_initial'] = array(
    '#type' => 'textfield',
    '#title' => t('Initial delay before polling starts, in seconds'),
    '#size' => 6,
    '#default_value' => variable_get('sqrl_poll_interval_initial', SQRL_DEFAULT_POLL_INTERVAL_INITIAL),
  );
  $form['options']['sqrl_poll_interval'] = array(
    '#type' => 'textfield',
    '#title' => t('Delay between polling requests, in seconds'),
    '#size' => 6,
    '#default_value' => variable_get('sqrl_poll_interval', SQRL_DEFAULT_POLL_INTERVAL),
  );

  $form['#validate'][] = 'sqrl_admin_validate_settings';
  $form['#submit'][] = 'sqrl_admin_submit_settings';

  return system_settings_form($form);
}

/**
 * Form validation handler for sqrl_admin().
 *
 * @param $form
 * @param $form_state
 *
 * @see sqrl_admin()
 */
function sqrl_admin_validate_settings($form, &$form_state) {
  // Validate the pattern for user names.
  $name = str_replace('[rand]', drupal_random_key($form_state['values']['sqrl_accounts_name_random_length']), $form_state['values']['sqrl_accounts_name_pattern']);
  if ($error = user_validate_name($name)) {
    form_error($form['sqrl_accounts_name_pattern'], $error);
  }
}

/**
 * Form submit handler for sqrl_admin().
 *
 * @param $form
 * @param $form_state
 *
 * @see sqrl_admin()
 */
function sqrl_admin_submit_settings($form, &$form_state) {
  if (variable_get('sqrl_session_allow_multiple', SQRL_DEFAULT_SESSION_ALLOW_MULTIPLE) != $form_state['values']['sqrl_session_allow_multiple']) {
    // Settings for multiple sessions per user have changed.
    // If new setting is ON: make users aware of the changed setting so that
    //    they can individually overwrite that setting for their account.
    // If new setting is OFF: make users aware of the changed setting and
    //    destroy all other sessions the next time the user accesses the site.
    if (empty($form_state['values']['sqrl_session_allow_multiple'])) {
      // New setting is OFF.
      _sqrl_remove_messages('multiple sessions turned on');
      _sqrl_add_message('multiple sessions turned off');
    }
    else {
      // New setting is ON.
      _sqrl_remove_messages('multiple sessions turned off');
      _sqrl_add_message('multiple sessions turned on');
    }
  }

  if (variable_get('sqrl_account_allow_multiple_per_id', SQRL_DEFAULT_ACCOUNT_ALLOW_MULTIPLE_PER_ID) != $form_state['values']['sqrl_account_allow_multiple_per_id']) {
    // Settings for multiple accounts per ID have changed.
    // If new setting is ON: make users aware of the changed setting so that
    //    they can individually overwrite that setting for their account.
    // If new setting is OFF: TODO: not sure yet what could be done!
    if (empty($form_state['values']['sqrl_session_allow_multiple'])) {
      // New setting is OFF.
    }
    else {
      // New setting is ON.
      _sqrl_remove_messages('multiple accounts turned off');
      _sqrl_add_message('multiple accounts turned on');
    }
  }

  if (variable_get('sqrl_id_allow_multiple_per_account', SQRL_DEFAULT_ID_ALLOW_MULTIPLE_PER_ACCOUNT) != $form_state['values']['sqrl_id_allow_multiple_per_account']) {
    // Settings for multiple IDs per account have changed.
    // If new setting is ON: make users aware of the changed setting so that
    //    they can individually overwrite that setting for their account.
    // If new setting is OFF: TODO: not sure yet what could be done!
    if (empty($form_state['values']['sqrl_session_allow_multiple'])) {
      // New setting is OFF.
    }
    else {
      // New setting is ON.
      _sqrl_remove_messages('multiple ids turned off');
      _sqrl_add_message('multiple ids turned on');
    }
  }

}
