<?php

/**
 * @file
 * Documentation for SQRL API.
 */

/**
 * Provides extra operations that this module supports and wants SQRL to be
 * used for authentication.
 *
 * @return array
 */
function hook_sqrl_supported_operations() {
  return array(
    'my_operation_1',
    'my_operation_2',
  );
}

/**
 * Determine if the current user has access to the given operation.
 *
 * @param string $op
 * @param string $mode
 *   Either 'user' or 'client'
 * @return bool
 *   If the given $op is provided by this module, returns either TRUE or FALSE
 *   depending on the permissions. If $op is not provided by this module then
 *   return nothing or NULL.
 */
function hook_sqrl_access($op, $mode) {
  switch ($op) {
    case 'my_operation_1':
      return ($mode == 'user');

    case 'my_operation_2':
      return FALSE;

  }
  return NULL;
}

/**
 * This hook gets called if a nut got successfully authenticated by a SQRL
 * client for the user $uid. It may execute some additional logic if the given
 * operation is provided by this module.
 *
 * @param string $op
 * @param int $uid
 */
function hook_sqrl_validation($op, $uid) {
  switch ($op) {
    case 'my_operation_1':
      // Do something for user $uid
      user_load($uid);
      break;

    case 'my_operation_2':
      // Do something else for user $uid
      break;

  }
}

/**
 * This hook gets called if the operation got authenticated by SQRL for the user
 * given in $uid and shall execute some code if required and return a URL to
 * which the user should be redirected afterwards.
 *
 * @param string $op
 * @param int $uid
 * @return string
 *   The URL to which the user should be redirected or nothing, if the given
 *   operation is not provided by this module.
 */
function hook_sqrl_perform($op, $uid) {
  switch ($op) {
    case 'my_operation_1':
      // Do something for user $uid
      user_load($uid);
      return '<front>';

    case 'my_operation_2':
      // Do something else for user $uid
      return 'checkout/completed';

  }
  return NULL;
}
